#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include "sqlite3.h"

#define MAX_SIZE        100
#define MODULE_SIZE     16

int displayTable(sqlite3_stmt *statementObject, DWORD columnIndex);

int wmain(void)
{
    /// SQL query for creating a table
    char createTable[MAX_SIZE] = "CREATE TABLE IF NOT EXISTS DemoTable (ID INTEGER,MODULE TEXT NOT NULL)";
    /// create variable for storing row data column name, column index, sql query, errorCheck
    char *tail = NULL;
    char module[MODULE_SIZE] = {0};
    char query[MAX_SIZE] = {0};
    BOOL columnCheck = TRUE;
    int choiceSelection = 0;
    int errorCheck;
    DWORD id = 0;
    DWORD modifiedId = 0;
    DWORD columnIndex = 0;
    DWORD queryCount = 5;
    DWORD querySize = 150;
    DWORD index = 0;
    sqlite3_stmt *statementObject;                                ///< A prepared statement for fetching tables
    sqlite3 *connection;                                          ///< create a database connection, create a pointer to sqlite3

    /// pointer to a query
    char **queries = malloc(sizeof(char) * queryCount * querySize);

    /// try to create the database. If it doesn't exist, it would be created
    /// pass a pointer to the pointer to sqlite3, in short sqlite3**
    errorCheck = sqlite3_open("sqliteDemo", &connection);

    /// if connection failed, connection returns NULL
    if (errorCheck)
    {
        printf("Database connection failed\n");
        return -1;
    }
    printf("SQLite Database Connection successful\n");

    errorCheck = sqlite3_exec(connection, createTable, 0, 0, 0);    ///< Execute the query for creating the table

    while (TRUE)
    {
        /// clear query and module for any garbage value
        memset(query, 0, sizeof(query));
        memset(module, 0, sizeof(module));

        /// SQLite query to select DemoTable from Database
        queries[index++] = "SELECT * from DemoTable";
        errorCheck = sqlite3_prepare(connection, queries[index-1], -1, &statementObject, 0);
        if (errorCheck)
        {
            printf("Selecting data from DB Failed\n");
            return -1;
        }
        printf("\n1. SHOW\n2. INSERT\n3. DELETE\n4. MODIFY\n5. SEARCH\n6. SORT\n7. EXIT \nEnter your choice ");
        scanf_s("%d", &choiceSelection);
        fflush(stdin);

        switch (choiceSelection)
        {
            case 1:
                /// show table
                displayTable(statementObject, columnIndex);
                break;
            case 2:
                /// insert data into table
                printf("Enter ID ");
                scanf_s("%d", &id);
                fflush(stdin);
                printf("Enter MODULE type ");
                scanf_s("%s", module);
                sprintf_s(query, MAX_SIZE, "INSERT INTO DemoTable VALUES(%d,'%s')", id, module);
                queries[index++] = query;
                errorCheck = sqlite3_exec(connection, queries[index-1], 0, 0, 0);
                if (errorCheck != SQLITE_OK)
                {
                    printf("ERROR in insertion");
                }
                else
                {
                    printf("\n***Inserted***\n");
                }
                errorCheck = displayTable(statementObject, columnIndex);
                break;
            case 3:
                /// delete from table
                printf("Enter ID to be Deleted ");
                scanf_s("%d", &id);
                sprintf_s(query, MAX_SIZE, "DELETE FROM DemoTable WHERE ID=%d", id);
                queries[index++] = query;
                errorCheck = sqlite3_exec(connection, queries[index-1], 0, 0, 0);
                if (errorCheck != SQLITE_OK)
                {
                    printf("ERROR in deletion");
                }
                else
                {
                    printf("\n***Deleted***\n");
                }
                displayTable(statementObject, columnIndex);
                break; 
            case 4:
                /// modify data in table
                printf("Enter ID to modify ");
                scanf_s("%d", &id);
                printf("Enter new id ");
                scanf_s("%d", &modifiedId);
                fflush(stdin);
                printf("Enter new module type ");
                fgets(module, sizeof(module), stdin);

                /// forming SQLite query for modify
                sprintf_s(query, MAX_SIZE, "UPDATE DemoTable SET  ID=%d,MODULE='%s' WHERE ID =%d", modifiedId, module, id);
                queries[index++] = query;
                errorCheck = sqlite3_exec(connection, queries[index-1], 0, 0, 0);
                if (errorCheck != SQLITE_OK)
                {
                    printf("ERROR in Modification");
                }
                else
                {
                    printf("\n***Modified***\n");
                }
                displayTable(statementObject, columnIndex);
                break;
            case 5:
                /// search 
                printf("Enter ID to search ");
                scanf_s("%d", &id);

                /// form SQLite query for search
                sprintf_s(query, MAX_SIZE, "Select * FROM DemoTable WHERE ID =%d", id);
                strcat_s(module, sizeof(module), "'");
                queries[index++] = query;
                errorCheck = sqlite3_prepare(connection, queries[index-1], -1, &statementObject, &tail);
                if (errorCheck != SQLITE_OK)
                {
                    printf("Error in searching");
                }

                /// search row
                while (sqlite3_step(statementObject) == SQLITE_ROW)
                {
                    if (columnCheck == TRUE)
                    {
                        printf("%s\t", sqlite3_column_name(statementObject, 0));
                        printf("%s\t\t", sqlite3_column_name(statementObject, 1));
                        printf("\n\n");
                    }
                    printf("%s\t", sqlite3_column_text(statementObject, 0));
                    printf("%s\n", sqlite3_column_text(statementObject, 1));
                }
                columnCheck = TRUE;
                break;
            case 6:
                /// SQLite query for sorting
                errorCheck = sqlite3_prepare(connection, "SELECT * FROM DemoTable ORDER BY ID", -1, &statementObject, &tail);
                if (errorCheck != SQLITE_OK)
                {
                    printf("ERROR");
                }

                /// to display sorted table
                while (sqlite3_step(statementObject) == SQLITE_ROW)
                {
                    if (columnCheck == TRUE)
                    {
                        /// fetch column name
                        printf("%s\t", sqlite3_column_name(statementObject, 0));
                        printf("%s\t\t", sqlite3_column_name(statementObject, 1));
                        printf("\n");
                    }
                    columnCheck = FALSE;
                    /// fetch row data
                    printf("%s\t", sqlite3_column_text(statementObject, 0));
                    printf("%s\n", sqlite3_column_text(statementObject, 1));
                }
                columnCheck = TRUE;
                break;
            case 7:
                /// exit
                exit(0);
            default:
                printf("Please enter valid choice\n");
                break;
        }
    } 
    /// close the connection to free memory
    sqlite3_close(connection);
    return 0;
}

/// @param[in] SQLite statementObject
/// @param[in] column index
/// @param[in] Connection pointer
int displayTable(sqlite3_stmt *statementObject, DWORD columnIndex)
{
    int errorCheck;
    BOOL columnCheck = TRUE;
    /// Read the number of rows fetched
    DWORD columnCount = sqlite3_column_count(statementObject);    
    printf("\n");
    while (TRUE)
    {
        /// fetch a row's status
        errorCheck = sqlite3_step(statementObject);

        /// SQLITE_ROW means fetched a row
        if (errorCheck == SQLITE_ROW)
        {
            /// sqlite3_column_text returns a constant void* , typecast it to constant char*
            for (columnIndex = 0 ; columnIndex < columnCount; columnIndex++)
            {
                if (columnCheck == TRUE)
                {
                    /// used to print column name
                    printf("%s\t", sqlite3_column_name(statementObject, 0));
                    printf("%s\t\t", sqlite3_column_name(statementObject, 1));
                    printf("\n");
                }
                columnCheck = FALSE;
                /// fetching row data from table
                printf("%s\t", sqlite3_column_text(statementObject, columnIndex));
            }
            printf("\n");
        }
        else if (errorCheck == SQLITE_DONE)
        {
            printf("\nAll rows fetched\n");
            break;
        }
        else
        {
            printf("Some error encountered\n");
            return -1;
        }
    }
    return -1;
}
